#
# p0f - passive OS fingerprinting
#
# Valid entry describes the way server starts TCP handshake (first SYN).
# Important options are: window size (wss), maximum segment size (mss),
# don't fragment flag (DF), window scaling (wscale), sackOK flag, nop
# flag, and initial time to live (TTL) ;)
#
# How can you determine initial ttl? Well, usually it's first power of 2
# bigger than TTL returned in scan. So, for example, if you get TTL 55 in
# fingerprint returned by p0f, initial TTL will be usually 64... NOTE:
# it's better to overestimate initial TTL than underestimate it ;)
#
# There are some brain-damaged devices, like network printers etc, that
# have stupid initial TTLs like 60, but who cares, if HP LaserJet wants to
# visit your server, you have to think again about your life ;)
#
# Format:
#
# wwww:ttt:mmm:D:W:S:N:OS Description
#
# wwww - window size
# ttt  - time to live
# mmm  - maximum segment size
# D    - don't fragment flag  (0=unset, 1=set) 
# W    - window scaling (-1=not present, other=value)
# S    - sackOK flag (0=unset, 1=set)
# N    - nop flag (0=unset, 1=set)
#
#
#
#  1xxx  - Windows
#  1000  - Windows (Unknown/Other)
#  1010  - Windows 9x
#  1011  - Windows 95
#  1012  - Windows 98
#  1013  - Windows ME
#  1020  - Windows NT
#  1021  - Windows NT 3.5
#  1022  - Windows NT 3.51
#  1023  - Windows NT 4.0
#  1030  - Windows 2000
#
#  2xxx  - Unix
#  2000  - Unix (Unknown/Other)
#  21xx  - Linux
#  2100  - Linux (Unknown/Other)
#  2101  - Linux 2.0.38
#  2102  - Linux 2.2.13
#  2103  - Linux 2.2.14
#  2104  - Linux 2.0.35
#  2105  - Linux 2.2.16
#  2106  - Linux 2.0.33
#  2107  - Linux 2.2.15
#  2108  - Linux 2.4.0
#  2110  - Linux Redhat
#  2111  - Linux Redhat 6.1
#  2120  - Linux Debian
#  22xx  - BSD
#  2200  - BSD (Unknown/Other)
#  2210  - BSD FreeBSD
#  2211  - BSD FreeBSD 4.0
#  2212  - BSD FreeBSD 2.2.6
#  2213  - BSD FreeBSD 2.2.8
#  2220  - BSD BSDI (Unknown/Other)
#  2221  - BSD BSDI 3.1
#  2230  - BSD NetBSD
#  2231  - BSD NetBSD 1.3
#  2240  - BSD OpenBSD
#  2241  - BSD OpenBSD 2.6
#  23xx  - HP/UX
#  2300  - HP/UX (Unknown/Other)
#  2301  - HP/UX 10
#  24xx  - AIX
#  2400  - AIX (Unknown/Other)
#  2401  - AIX 3
#  2402  - AIX 4
#  25xx  - IRIX
#  2500  - IRIX (Unknown/Other)
#  2510  - IRIX 6.x
#  2511  - IRIX 6.4
#  2512  - IRIX 6.5
#  2513  - IRIX 6.3
#  2514  - IRIX 5.3
#  26xx  - Solaris
#  2600  - Solaris (Unknown/Other)
#  2610  - Solaris 2.x series
#  2611  - Solaris 2.6
#  2612  - Solaris 2.7
#  2620  - Solaris 7.x series
#  27xx  - SCO
#  2700  - SCO (Unknown/Other)
#  2701  - SCO 7.0.1
#  2702  - SCO 7.1.0
#  2800  - Digital Unix (Unknown/Other)
#  2900  - Digital Unix (Unknown/Other)
#
#  3xxx  - Router
#  3000  - Router (Unknown/Other)
#  3100  - Cisco (Unknown/Other)
#  3101  - Cisco IOS 11
#  3102  - Cisco IOS 12
#
#  9xxx  - OTHER

1010:8192:128:1460:1:0:1:1:Windows 9x (1)
1010:8192:128:536:1:0:1:1:Windows 9x (2)
1011:5840:128:536:1:0:1:1:Windows 95 (3)
1011:8192:32:1456:1:-1:0:0:Windows 95 (?)
1010:2144:64:536:1:0:1:1:Windows 9x (4)
1020:8192:128:1460:1:-1:1:0:Windows NT 
1023:8192:128:1460:1:0:0:0:Windows NT 4.0
1023:8192:32:1460:1:0:0:0:Windows NT 4.0
1023:8192:128:25443:1:-1:1:1:Microsoft NT 4.0 Server SP5
1023:8192:64:1460:1:-1:0:0:AXCENT Raptor Firewall Windows NT 4.0/SP3
1030:16384:128:1460:1:0:1:1:Windows 2000
2101:512:64:1460:0:0:0:0:Linux 2.0.38
2101:32120:58:1460:0:-1:0:0:Linux 2.0.38
2102:32120:32:1460:1:0:1:1:Linux 2.2.13
2102:8192:128:1456:1:0:1:1:Linux 2.2.13
2103:32120:64:1460:1:0:1:1:Linux 2.2.14 or Cobalt Linux 2.2.12C3
2104:512:64:0:0:-1:0:0:Linux 2.0.35 - 2.0.37
2105:32120:64:1460:1:190:1:1:Linux 2.2.16
2106:32120:64:1460:0:-1:0:0:Linux 2.0.33
2106:512:64:1460:0:52:0:0:Linux 2.0.33
2107:32120:64:1460:1:101:1:1:Linux 2.2.15
2108:31856:64:1460:1:0:1:1:Linux 2.4.0-test1
2111:31072:64:3884:1:0:1:1:Linux 2.2.12-20 (RH 6.1)
2120:16060:64:1460:1:0:1:1:Debian/Caldera Linux 2.2.x (check)
2211:16384:64:1460:1:0:0:0:FreeBSD 4.0-STABLE, 3.2-RELEASE
2212:16384:255:1460:1:0:0:1:FreeBSD 2.2.6-RELEASE
2213:16384:64:1460:1:0:0:1:FreeBSD 2.2.8-RELEASE
2231:16384:64:1460:0:0:0:1:NetBSD 1.3/i386
2221:8192:64:1460:1:0:0:1:BSDI BSD/OS 3.1
2241:16384:64:512:0:0:0:1:OpenBSD 2.6
2301:32768:64:1460:1:0:0:0:HP-UX B.10.01 A 9000/712
2402:16384:64:512:0:0:0:0:AIX 3.2, 4.2 - 4.3
2512:49152:64:1460:0:0:0:0:IRIX 6.5 / 6.4
2513:61440:64:1460:0:-1:0:0:IRIX 6.3
2514:61440:64:512:0:-1:0:0:IRIX 5.3 / 4.0.5F
2611:8760:64:1460:1:0:0:0:Solaris 2.6 (2)
2611:9140:255:9140:1:0:0:0:Solaris 2.6 (sunsite)
2611:8760:255:1460:1:0:0:0:Solaris 2.6 or 2.7
2612:8760:255:1380:1:0:0:0:Solaris 2.7
2620:33580:255:1460:1:-1:0:0:Solaris 7
2701:24820:64:1460:1:0:0:0:SCO UnixWare 7.0.1
2702:32696:64:536:0:0:1:1:SCO UnixWare 7.1.0 x86
2702:24820:64:1460:1:0:0:1:SCO UnixWare 7.1.0 x86
2800:32768:64:1460:1:0:0:1:Digital UNIX V4.0E
2900:16384:64:0:0:-1:0:0:ULTRIX V4.5 (Rev. 47)
3101:4288:255:536:0:-1:0:0:Cisco 2500 IOS 11.2(5)P or Cisco 4500 IOS 11.1(7)
3101:2144:255:536:0:-1:0:0:Cisco IGS 3000 IOS 11.x(16), 2500 IOS 11.2(3)P
3101:4128:255:1460:0:-1:0:0:Cisco 2611 IOS 11.3(2)XA4
3101:4288:255:1460:0:-1:0:0:Cisco 3620 IOS 11.2(17)
3101:4288:255:536:0:-1:0:0:Cisco 1600 IOS 11.2(15)P
3102:4128:255:556:0:0:0:0:Cisco 1750 IOS 12.0(5), Cisco 2500 IOS 11.3(1)
3102:4128:255:556:0:0:0:0:Cisco 3600 IOS Version 12.0(7)
3102:4128:255:1460:0:-1:0:0:Cisco 4500 IOS 12.0(9), 3640 12.1(2), 3620 12.0(8) or 11.3(11a)
3102:4128:255:1460:0:-1:0:0:Cisco C2600 IOS 12.0(5)T1
9000:32768:64:1432:0:0:0:0:??? (PlusGSM, InterNetia proxy)
9000:4096:30:1024:0:245:0:0:Alcatel (Xylan) OmniStack 5024 v3.4.5
9000:4096:32:1024:0:245:0:0:Alcatel (Xylan) OmniStack 5024
9000:32768:128:1460:1:-1:0:0:Novell NetWare 4.11
9000:32694:255:536:0:0:0:0:3Com HiPer ARC, System V4.2.32
9000:65535:128:1368:1:-1:0:0:BorderManager 3.5
9000:4096:30:1024:0:-1:0:0:Chorus MiX V.3.2 r4.1.5 COMP-386
