#include "sp_host_id.h"
#include "spp_host_id.h"

extern char *file_name;  /* this is the file name from rules.c, generally used
                            for error messages */

extern int file_line;    /* this is the file line number from rules.c that is
                            used to indicate file lines for error messages */

#define RANGE_FILE_NAME	"spp_host_id.range"
							
/****************************************************************************
*
****************************************************************************/
void SetupSpHostID()
{
    /* map the keyword to an initialization/processing function */
    RegisterPlugin("host_id", SpHostIDInit);

#ifdef DEBUG
    printf("Plugin: host_id Initialized\n");
#endif
}


/****************************************************************************
*
****************************************************************************/
void SpHostIDInit(char *data, OptTreeNode *otn, int protocol)
{

#ifdef DEBUG
	printf("Initializing Host_ID Plugin\n");
#endif

    /* allocate the data structure and attach it to the
       rule's data struct list */
    otn->ds_list[PLUGIN_HOSTID] = (SpHostIDData *) calloc(sizeof(SpHostIDData), sizeof(char));

    /* this is where the keyword arguments are processed and placed into the 
       rule option's data structure */
    ParseSpHostID(data, otn);

    /* finally, attach the option's detection function to the rule's 
       detect function pointer list */
    AddOptFuncToList(HostIDCheckEq, otn);
}



/****************************************************************************
*
****************************************************************************/
void ParseSpHostID(char *data, OptTreeNode *otn)
{
	FILE*			fp;
	char			Buff[1024];
	char*			next;
	char*			end;
	int				count;
	int				round;
	unsigned short	min=0;
	unsigned short	max=0;
	
    SpHostIDData 	*ds_ptr;  /* data struct pointer */

    /* set the ds pointer to make it easier to reference the option's
       particular data struct */
    ds_ptr = otn->ds_list[PLUGIN_HOSTID];

    /* get rid of any whitespace */
    while(isspace((int)*data))
    {
        data++;
    }

	/*set min and max here*/	
	fp=fopen(RANGE_FILE_NAME, "r");
	count=0;
	
	bzero(Buff,1024);
	fgets(Buff, 1024, fp);
	while (Buff[0]!=0){
		if ( (Buff[0]!=(char)"#"[0]) && (Buff[0]!=(char)" "[0])){
			next=Buff;
			end=Buff;
			round=0;
			while (next && end){
				end=strchr(next, (int)":"[0]);
				if (!end) continue;
				end[0]=0;
				switch (round){
				case 0:
					min=atoi(next);
					break;
				case 1:
					max=atoi(next);
					next=end+1;
				case 2:
					if (strncasecmp(data, next, strlen(next)-1)==0){
						ds_ptr->min=min;
						ds_ptr->max=max;
#ifdef DEBUG
						printf("Found %s\n",next);
#endif
					}
					continue;
				}
				round++;
				next=end+1;
			}
			count++;
		}
		bzero(Buff,1024);
		fgets(Buff, 1024, fp);
	}
	fclose(fp);
}


/****************************************************************************
*
****************************************************************************/
int HostIDCheckEq(Packet *p, struct _OptTreeNode *otn, OptFpList *fp_list)
{
	host_rec*	this;
	/*lookup that IP*/
	this=node_head;
	while(this){
		if (memcmp(&this->IP, &p->iph->ip_dst, sizeof(struct in_addr))==0)
			break;
		this=this->next;
	}
	
	if (!this) return 0;
	
	/*check to see if it's the right one*/
    if(
	      (((SpHostIDData *)otn->ds_list[PLUGIN_HOSTID])->min <= this->host_type) &&
		  (((SpHostIDData *)otn->ds_list[PLUGIN_HOSTID])->max >= this->host_type)
	  )
    {
#ifdef DEBUG
		printf("This one matches\n");
		printf("%s is %u\n",inet_ntoa(p->iph->ip_dst),this->host_type);
#endif
        /* call the next function in the function list recursively */
        return fp_list->next->OptTestFunc(p, otn, fp_list->next);
    }
#ifdef DEBUG
    else
    {
        /* you can put debug comments here or not */
        printf("No match\n");
    }
#endif

    /* if the test isn't successful, return 0 */
    return 0;
}
