#include "spp_host_id.h"
/****************************************************************
* spp_host_id
*
* performs passive host detection on local network
*
*
****************************************************************/
#include <time.h>

/*+++++++++++++++++++++++++++Global Variables++++++++++++++++++*/
#define	NUM_SIGS	60000
#define EXPIRE_TIME	2
#define SIG_FILE_NAME	"p0f.fp"

sig_rec		sigs[NUM_SIGS];
char*		Desc[NUM_SIGS];


/*++++++++++++++++++++++++++Begin Code+++++++++++++++++++++++++*/
void AddHostIDNode(struct in_addr IP, unsigned short type, struct timeval t);

/*************************************************************
* Load the signatures from the sig file
*
* I'm really bad at writing parsers so if anyone wants to 
* rewrite this...
*************************************************************/
void LoadSigs(){
	FILE*	fp;
	char	Buff[1024];
	char*	next;
	char*	end;
	int		count;
	int		round;
	
	fp= fopen(SIG_FILE_NAME, "r");
	count=0;
	
	bzero(Buff,1024);
	fgets(Buff, 1024, fp);
	while (Buff[0]!=0){
		if ( (Buff[0]!=(char)"#"[0]) && (Buff[0]!=(char)" "[0])){
			next=Buff;
			end=Buff;
			round=0;
			while (next && end){
				end=strchr(next, (int)":"[0]);
				if (!end) continue;
				end[0]=0;
				switch (round){
				case 0:
					sigs[count].ID=atoi(next);
					break;
				case 1:
					sigs[count].WindowSize=atoi(next);
					break;
				case 2:
					sigs[count].TTL=atoi(next);
					break;
				case 3:
					sigs[count].MaxSegSize=atoi(next);
					break;
				case 4:
					if (atoi(next))
					  sigs[count].BitFlags = sigs[count].BitFlags | SIG_DF;
					break;
				case 5:
					sigs[count].WindowScaling=atoi(next);
					break;
				case 6:
					if (atoi(next))
					  sigs[count].BitFlags = sigs[count].BitFlags | SIG_SACKOK;
					break;	
				case 7:
					if (atoi(next))
					  sigs[count].BitFlags = sigs[count].BitFlags | SIG_NOP;
					//strcpy(sigs[count].Desc, &end[1]);
					Desc[count]=malloc(255);
					strcpy(Desc[count],&end[1]);
					continue;
				}
				round++;
				next=end+1;
			}
			if (sigs[count].WindowSize) count++;
		}
		bzero(Buff,1024);
		fgets(Buff, 1024, fp);
	}
	fclose(fp);
}

/*************************************************************
* Initialize the global variables
*************************************************************/
void host_id_init_vars(){
	node_head=NULL;
	bzero(sigs, sizeof(sig_rec) * NUM_SIGS);
}

/************************************************************
* Standard setup call
************************************************************/
void SetupHostID(void){
	RegisterPreprocessor("host_id", HostIDInit);
}

/*************************************************************
* Standard init call
*************************************************************/
void HostIDInit(u_char* args){
	host_id_init_vars();
	LoadSigs();
	AddFuncToPreprocList(HostIDPreprocFunction);
}

/*************************************************************
* Guess initial TTL and hops
*************************************************************/
unsigned char GuessTTL(unsigned char c){
	/*common TTL's*/
	if (c==255) return 255;
	if (c==128) return 128;
	if (c==64) return 64;
	if (c==32) return 32;
	
	if (c>128) return 255;
	if (c>64) return 128;
	if (c>32) return 64;

	return 32;
}

/************************************************************
* Called with every packet
************************************************************/
void HostIDPreprocFunction(Packet* p){

	sig_rec	this;
	int		i;

	if (!p->iph) return;
	if (!p->tcph) return;
	if ((p->tcph->th_flags & TH_SYN)!=TH_SYN) return;

	/*we have a syn, decode it*/
	bzero(&this, sizeof(sig_rec));
	this.WindowSize=ntohs(p->tcph->th_win);
	this.TTL=GuessTTL(p->iph->ip_ttl);
	this.MaxSegSize=0; 			/*lookup*/
	this.WindowScaling=0;		/*lookup*/
	if ((p->df)==1) this.BitFlags = this.BitFlags | SIG_DF;
	for (i=0;i<p->tcp_option_count;i++){
		switch (p->tcp_options[i].code){
		case TOPT_NOP:
			this.BitFlags = this.BitFlags | SIG_NOP;
			break;
		case TCPOPT_SACKOK:
			this.BitFlags = this.BitFlags | SIG_SACKOK;
			break;	
		case TCPOPT_WSCALE:
			this.WindowScaling=*(u_int8_t*)p->tcp_options[i].data;
			break;	
		case TOPT_MSS:
			this.MaxSegSize=ntohs(*(u_int16_t*)p->tcp_options[i].data);
			break;	
		}
	}
	
	/*printf("%s->%lu:%u:%u:%u:%i:%u:%u\n",
		inet_ntoa(p->iph->ip_src),
		this.WindowSize,
		this.TTL,
		this.MaxSegSize,
		this.BitFlags & SIG_DF,
		this.WindowScaling,
		(this.BitFlags & SIG_SACKOK)/SIG_SACKOK,
		(this.BitFlags & SIG_NOP)/SIG_NOP);*/
			
	for (i=0;i<NUM_SIGS;i++){
		if (memcmp(&this, &sigs[i], sizeof(sig_rec)-sizeof(short))==0){
			AddHostIDNode(p->iph->ip_src, sigs[i].ID, *(struct timeval*)&p->pkth->ts);
#ifdef DEBUG
			printf("%s is %s",inet_ntoa(p->iph->ip_src),Desc[i]);
#endif
			break;
		}
	}
}

/*********************************************************
* Replace this with something faster
* This is new. I send the info to a database
*********************************************************/
void AddHostIDNode(struct in_addr IP, unsigned short type, struct timeval t){
	host_rec*	this;
	host_rec*	del;
	host_rec	new_rec;
	
	new_rec.IP=IP;
	new_rec.host_type=type;
	new_rec.last_time=t;
	new_rec.next=NULL;
	
	if (!node_head){
		node_head=(host_rec*)malloc(sizeof(host_rec));
		memcpy(node_head, &new_rec, sizeof(host_rec));
#ifdef DEBUG
		printf("First node %s\n",inet_ntoa(IP));
#endif
		return;
	}
	
	if (memcmp(&node_head->IP, &IP, sizeof(struct in_addr))==0){
		node_head->last_time=t;
		return;
	}
	
	this=node_head;
	while (this->next){
		if (memcmp(&this->next->IP, &IP, sizeof(struct in_addr))==0){
			this->next->last_time=t;
			return;
		}
		if ( (t.tv_sec-this->next->last_time.tv_sec) > EXPIRE_TIME ){
#ifdef DEBUG
			printf("Deleting old record %s\n", inet_ntoa(this->next->IP));
#endif
			/*del=this;
			while (del){
				printf("->%s",inet_ntoa(del->IP));
				del=del->next;
			}
			printf("\n");*/

			del=this->next;
			if (this->next && this->next->next) this->next = this->next->next;
			free(del);
		}
		this=this->next;
	}
	
	this->next=(host_rec*)malloc(sizeof(host_rec));
	memcpy(this->next, &new_rec, sizeof(host_rec));
#ifdef DEBUG
	printf("New %s\n",inet_ntoa(IP));
#endif
}
