#include "spp_uni_scrub.h"
#include "unidecode.h"

#define MODNAME "spp_uni_scrub"

void SetupUniScrub()
{
    RegisterPreprocessor("uni_scrub", UniScrubInit);

#ifdef DEBUG
    printf("Preprocessor: UniScrub in setup...\n");
#endif
}


void UniScrubInit(u_char *args)
{
    init_codes();   
    /* Set the preprocessor function into the function list */
    AddFuncToPreprocList(PreprocUniScrub);
}

int SlashPad(char* OutBuff, int TargetLen, char*InBuff,int InLen){
	int SlashPos=0;
	int i;
	
	//printf("Padding %i slashes\n",TargetLen-InLen-1);
	if (TargetLen<=InLen) return TRUE;
	
	/*find a slash*/
	for (i=0;i<InLen;i++){
		if ( (InBuff[i]==0x2f) || (InBuff[i]==0x5c) ){
			SlashPos=i; 
			break;
		}
	}
	
	if (i>=InLen){
		printf("Couldn't find a slash\n");
		return FALSE;
	}
	
	/*copy before the slashes*/
	memcpy(OutBuff, InBuff, SlashPos+1);
	
	/*pad slashes*/
	for (i=0; i<(TargetLen-InLen); i++)
		OutBuff[SlashPos+i+1]=0x5c;
		
	/*copy the stuff after the slashes*/
	memcpy(OutBuff+SlashPos+(TargetLen-InLen), InBuff+SlashPos+1, (TargetLen-SlashPos)); 

}

void PreprocUniScrub(Packet *p)
{
    struct pseudoheader       /* pseudo header for TCP checksum calculations */
    {
        u_int32_t sip, dip;   /* IP addr */
        u_int8_t  zero;       /* checksum placeholder */
        u_int8_t  protocol;   /* protocol number */
        u_int16_t tcplen;     /* tcp packet length */
    };
    struct pseudoheader ph;    /* pseudo header declaration */
	unsigned int hlen;
	unsigned int ip_len;

	char TestBuff[4096];	
	char BeforeBuff[4096];
	int newsize;
	

	if (!p->iph) return;
	if (!p->tcph) return;
	if (p->dp!=80) return;
	if (!p->data) return;
	if (p->dsize==0) return;
	
	
	bzero(BeforeBuff,4096);
	memcpy(&BeforeBuff[0], p->data, p->dsize);

	newsize=TranslateUnicode(BeforeBuff ,p->dsize, TestBuff, 4096);
	memcpy(BeforeBuff, TestBuff, newsize+1);
	newsize=TranslateUnicode(BeforeBuff ,newsize, TestBuff, 4096);
	SlashPad(p->data, p->dsize, TestBuff, newsize);
	
	/*update the checksums*/
	p->iph->ip_csum=0;
	hlen = p->iph->ip_hlen << 2;
	p->iph->ip_csum=checksum((u_short *)p->iph, hlen, (u_short *)NULL, 0);
	//printf("New IP csum %04x\n",p->iph);

	ip_len=ntohs(p->iph->ip_len);
	ip_len -= hlen;
	
	p->tcph->th_sum=0;
    hlen = p->tcph->th_off << 2;
    ph.sip = (u_int32_t)(p->iph->ip_src.s_addr);
    ph.dip = (u_int32_t)(p->iph->ip_dst.s_addr);
    ph.zero = 0;
    ph.protocol = p->iph->ip_proto;
    ph.tcplen = htons(ip_len);
    p->tcph->th_sum = checksum((u_int16_t *)&ph, 12, (u_int16_t *)(p->tcph), ip_len);
	//printf("TCP csum %04x\n",p->tcph->th_sum);
	

	/*bzero(BeforeBuff, 4096);
	SlashPad(BeforeBuff, p->dsize, TestBuff, newsize);	
	printf("%s\n",BeforeBuff);
	if (p->dsize==strlen(BeforeBuff)) printf("Sizes Match\n");*/
	
	
	
}




