#include "unidecode.h"
/************************************************************
* unidecode.c
*
* Decodes unicode encoded characters and puts them in the
* 1 byte ascii translation that MS IIS uses them as.  This
* is by no means a complete mapping, but it is designed to 
* be extensible as mappings are found.
*
* Notes:  Only decode up to 3 byte right now
*************************************************************/
#include <stdlib.h>

/*function prototypes*/
int GetNextChar(char* Buff, char* NextChar);

/*************************************************************
* Do the Translation. Return the new buffer size
* returns the size of the new buffer or -1 on failure
*************************************************************/
int TranslateUnicode(char* InBuff, int InLength, char* OutBuff, int OutLength){	
	int	i;
	int	out_pos;
	int	num_this_char;
	char	new_char;
	int		seen_question;

#ifdef DEBUG
	printf("Decoding\n");
#endif
	
	bzero(OutBuff, OutLength);	
	out_pos=0;
	i=0;
	seen_question=FALSE;
	while ( TRUE ){
		
		//printf("Output %s\n",OutBuff);
		
		if (InBuff[i]==63) seen_question=TRUE;
		
		if (seen_question){
			/*don't decode anything after a question mark - CGI stuff*/
			num_this_char=1;
			if (out_pos!=OutLength){
				OutBuff[out_pos++]=InBuff[i];
			}else{
				OutBuff[out_pos]=0;
				return -1;
			}
		}else{
			/*normal decode*/
			num_this_char=GetNextChar(&InBuff[i], &new_char);
			if (num_this_char==0) break;
		
			if (out_pos!=OutLength){
				OutBuff[out_pos++]=new_char;
			}else{
				OutBuff[out_pos]=0;
				return -1;
			}
		}
			
		i=i+num_this_char;
	
		if (i>InLength){
			return out_pos-1; 
		}
	}
	
	return out_pos-1;
}


/*******************************************
* returns the number of bytes used in the 
* IN buff to make the character
*******************************************/
int GetNextChar(char* Buff, char* NextChar){
	char		Code[3];
	unsigned long 	Hex;
	unsigned long 	Hex2;
	unsigned long 	Hex3;

	/*defaults to a question mark*/
	//NextChar[0]=0x3f;
	NextChar[0]=0x25;

	if (Buff[0]==0){
		return 0;
	}
	
	if (Buff[0]==0x25){
		Code[0]=Buff[1];
		Code[1]=Buff[2];
		Code[2]=0;
		Hex = strtoul(Code, NULL, 16);
		
		if ((Hex & 128)==0){
#ifdef DEBUG
			printf("1 Byte\n");
#endif
			if (codes[Hex]!=0){
				NextChar[0] = codes[Hex];
				return 3;
			}else{
#ifdef DEBUG
				printf("Not a Hex Code\n");
#endif
				NextChar[0]=Buff[0];
				return 1;
			}
		}else if ((Hex & 224) == 192){
			printf("2 Byte\n");
			if (Buff[3]!=0x25){
#ifdef DEBUG
				printf("Invalid UTF-8 Code\n");
#endif
				NextChar[0]=0x25;
				return 1;
			}
			
			Code[0]=Buff[4];
			Code[1]=Buff[5];
			Code[2]=0;
			Hex2 = strtoul(Code, NULL, 16);
			
			if ((Hex2 & 192)!=128){
#ifdef DEBUG
				printf("Second Byte Invalid\n");
#endif 
				NextChar[0]=0x25;
				return 1;
			}
			
			Hex = Hex & 31;
			Hex2 = Hex2 & 63;	
			NextChar[0] = codes[(Hex * 64) | Hex2];

			return 6;
		}else if ((Hex & 240) == 224){
#ifdef DEBUG
			printf("3 Byte\n");
#endif
			if (Buff[3]!=0x25){
				printf("Invalid UTF-8 Code\n");
				NextChar[0]=0x25;
				return 1;
			}
			if (Buff[6]!=0x25){
#ifdef DEBUG
				printf("Invalid UTF-8 Code\n");
#endif
				NextChar[0]=0x25;
				return 1;
			}
			
			Code[0]=Buff[4];
			Code[1]=Buff[5];
			Code[2]=0;
			Hex2 = strtoul(Code, NULL, 16);
			
			if ((Hex2 & 192)!=128){
				printf("Second Byte Invalid\n");
				NextChar[0]=0x25;
				return 1;
			}
			
			Code[0]=Buff[7];
			Code[1]=Buff[8];
			Code[2]=0;
			Hex3 = strtoul(Code, NULL, 16);
			
			if ((Hex3 & 192)!=128){
				printf("Second Byte Invalid\n");
				NextChar[0]=0x25;
				return 1;
			}
			
			Hex = Hex & 15;
			Hex2 = Hex2 & 63;	
			Hex3 = Hex3 & 63;	
			NextChar[0] = codes[(Hex * 4096) | (Hex2 * 64) | Hex3];
#ifdef DEBUG			
			printf("Final Code %02x\n",(Hex * 4096) | (Hex2 * 64) | Hex3);
#endif
			return 9;
		}
		
	}else{
		NextChar[0] = Buff[0];
	}

	return 1;
}
